@@ -1,14 +1,15 @@
 // SPDX-License-Identifier: Apache-2.0
-
 use std::collections::HashMap;
 use std::ffi::OsStr;
-use std::fs::OpenOptions;
+use std::fs;
+use std::os::unix::fs::FileTypeExt;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
 use walkdir::WalkDir;
 
 const CRATE: &str = env!("CARGO_MANIFEST_DIR");
 const TEST_BINS_IN: &str = "tests/c-tests";
+const AESM_SOCKET: &str = "/var/run/aesmd/aesm.socket";
 
 fn find_files_with_extensions<'a>(
     exts: &'a [&'a str],
@@ -81,6 +82,20 @@ fn cargo_build_bin(
     target_name: &str,
     bin_name: &str,
 ) -> std::io::Result<()> {
+    // And here's where we'd like to place the final (stripped) binary
+    let out_bin = out_dir.join("bin").join(bin_name);
+
+    // Don't run the build if ENARX_PREBUILT_${bin_name} is set
+    let prebuilt_env_name = format!("ENARX_PREBUILT_{}", bin_name);
+    if let Ok(prebuilt_path) = std::env::var(&prebuilt_env_name) {
+        println!(
+            "cargo:warning=Using prebuilt {} binary from {}: {}",
+            bin_name, prebuilt_env_name, &prebuilt_path
+        );
+        std::fs::copy(prebuilt_path, out_bin)?;
+        return Ok(());
+    }
+
     let profile: &[&str] = match std::env::var("PROFILE").unwrap().as_str() {
         "release" => &["--release"],
         _ => &[],
@@ -115,13 +130,13 @@ fn cargo_build_bin(
 
     let target_dir = out_dir.join(path);
 
-    let stdout: Stdio = OpenOptions::new()
+    let stdout: Stdio = fs::OpenOptions::new()
         .write(true)
         .open("/dev/tty")
         .map(Stdio::from)
         .unwrap_or_else(|_| Stdio::inherit());
 
-    let stderr: Stdio = OpenOptions::new()
+    let stderr: Stdio = fs::OpenOptions::new()
         .write(true)
         .open("/dev/tty")
         .map(Stdio::from)
@@ -163,9 +178,6 @@ fn cargo_build_bin(
         .join(std::env::var("PROFILE").unwrap())
         .join(bin_name);
 
-    // And here's where we'd like to place the final (stripped) binary
-    let out_bin = out_dir.join("bin").join(bin_name);
-
     // Strip the binary
     let status = Command::new("strip")
         .arg("--strip-unneeded")
@@ -240,7 +252,7 @@ fn main() {
             "wasmldr" => cargo_build_bin(&path, &out_dir, target, "wasmldr").unwrap(),
 
             #[cfg(feature = "backend-kvm")]
-            "shim-sev" => cargo_build_bin(&path, &out_dir, target, "shim-sev").unwrap(),
+            "shim-kvm" => cargo_build_bin(&path, &out_dir, target, "shim-kvm").unwrap(),
 
             #[cfg(feature = "backend-sgx")]
             "shim-sgx" => cargo_build_bin(&path, &out_dir, target, "shim-sgx").unwrap(),
@@ -252,4 +264,30 @@ fn main() {
             std::fs::remove_file(&cargo_toml).unwrap()
         }
     }
+
+    if std::path::Path::new("/dev/sgx_enclave").exists()
+        && fs::metadata("/dev/sgx_enclave")
+            .unwrap()
+            .file_type()
+            .is_char_device()
+    {
+        println!("cargo:rustc-cfg=host_can_test_sgx");
+
+        if std::path::Path::new(AESM_SOCKET).exists()
+            && fs::metadata(AESM_SOCKET).unwrap().file_type().is_socket()
+        {
+            println!("cargo:rustc-cfg=host_can_test_attestation");
+        }
+    }
+
+    if std::path::Path::new("/dev/sev").exists() {
+        // Not expected to fail, as the file exists.
+        let metadata = fs::metadata("/dev/sev").unwrap();
+        let file_type = metadata.file_type();
+
+        if file_type.is_char_device() {
+            println!("cargo:rustc-cfg=host_can_test_sev");
+            println!("cargo:rustc-cfg=host_can_test_attestation");
+        }
+    }
 }
