@@ -18,7 +18,6 @@
 pub mod entry;
 pub mod handler;
 pub mod heap;
-pub mod uarch;
 
 use sgx::parameters::{Attributes, Features, MiscSelect, Xfrm};
 
@@ -50,6 +49,9 @@ pub const MISC: MiscSelect = {
     }
 };
 
+/// The size of the sallyport block
+pub const BLOCK_SIZE: usize = 69632;
+
 // NOTE: You MUST take the address of these symbols for them to work!
 extern "C" {
     /// Extern
