@@ -1,10 +1,11 @@
 // SPDX-License-Identifier: Apache-2.0
 
+use super::config::Config;
 use super::mem::Region;
 use super::KvmKeepPersonality;
 
 use std::convert::TryFrom;
-use std::mem::size_of;
+use std::mem::align_of;
 use std::sync::{Arc, RwLock};
 
 use anyhow::{Context, Error, Result};
@@ -13,10 +14,10 @@ use kvm_bindings::fam_wrappers::KVM_MAX_CPUID_ENTRIES;
 use kvm_ioctls::{Kvm, VcpuFd, VmFd};
 use mmarinus::{perms, Map};
 use sallyport::elf::pf::kvm::SALLYPORT;
-use sallyport::Block;
-use x86_64::VirtAddr;
+use x86_64::{align_up, VirtAddr};
 
 pub struct Builder {
+    config: Config,
     kvm_fd: Kvm,
     vm_fd: VmFd,
     regions: Vec<Region>,
@@ -26,13 +27,14 @@ pub struct Builder {
 impl TryFrom<super::config::Config> for Builder {
     type Error = Error;
 
-    fn try_from(_config: super::config::Config) -> Result<Self> {
+    fn try_from(config: Config) -> Result<Self> {
         let kvm_fd = Kvm::new().context("Failed to open '/dev/kvm'")?;
         let vm_fd = kvm_fd
             .create_vm()
             .context("Failed to create a virtual machine")?;
 
         Ok(Builder {
+            config,
             kvm_fd,
             vm_fd,
             regions: Vec::new(),
@@ -57,6 +59,7 @@ impl super::super::Mapper for Builder {
         }
 
         let mem_region = kvm_builder_map(
+            self.config.sallyport_block_size,
             &mut self.sallyports,
             &mut self.vm_fd,
             &mut pages,
@@ -72,6 +75,7 @@ impl super::super::Mapper for Builder {
 }
 
 pub fn kvm_builder_map(
+    block_size: usize,
     sallyports: &mut Vec<Option<VirtAddr>>,
     vm_fd: &mut VmFd,
     pages: &mut Map<perms::ReadWrite>,
@@ -80,8 +84,9 @@ pub fn kvm_builder_map(
     slot: u32,
 ) -> anyhow::Result<kvm_userspace_memory_region> {
     if with & SALLYPORT != 0 {
-        for start in (0..pages.size()).step_by(size_of::<Block>()) {
-            if start + size_of::<Block>() <= pages.size() {
+        for start in (0..pages.size()).step_by(block_size) {
+            let start = align_up(start as u64, align_of::<usize>() as u64) as usize;
+            if start + block_size <= pages.size() {
                 let virt = VirtAddr::from_ptr(pages.as_ptr()) + start;
                 sallyports.push(Some(virt));
             }
@@ -108,7 +113,7 @@ impl TryFrom<Builder> for Arc<dyn super::super::Keep> {
     type Error = Error;
 
     fn try_from(mut builder: Builder) -> Result<Self> {
-        let (vcpu_fd, sallyport_block_start) =
+        let vcpu_fd =
             kvm_try_from_builder(&builder.sallyports, &mut builder.kvm_fd, &mut builder.vm_fd)?;
 
         Ok(Arc::new(RwLock::new(super::Keep::<KvmKeepPersonality> {
@@ -116,8 +121,8 @@ impl TryFrom<Builder> for Arc<dyn super::super::Keep> {
             vm_fd: builder.vm_fd,
             cpu_fds: vec![vcpu_fd],
             regions: builder.regions,
+            sallyport_block_size: builder.config.sallyport_block_size,
             sallyports: builder.sallyports,
-            sallyport_start: sallyport_block_start,
             personality: KvmKeepPersonality(()),
         })))
     }
@@ -127,7 +132,7 @@ pub fn kvm_try_from_builder(
     sallyports: &[Option<VirtAddr>],
     kvm_fd: &mut Kvm,
     vm_fd: &mut VmFd,
-) -> Result<(VcpuFd, VirtAddr)> {
+) -> Result<VcpuFd> {
     // If no LOAD segment were defined as sallyport blocks
     if sallyports.is_empty() {
         anyhow::bail!("No sallyport blocks defined!");
@@ -144,8 +149,5 @@ pub fn kvm_try_from_builder(
         .set_cpuid2(&cpuids)
         .context("Failed to set the supported CPUID entries")?;
 
-    // FIXME: this will be removed with relative addresses in sallyport
-    // unwrap, because we have at least one block
-    let sallyport_block_start = sallyports.first().unwrap().unwrap();
-    Ok((vcpu_fd, sallyport_block_start))
+    Ok(vcpu_fd)
 }
