@@ -36,8 +36,7 @@ pub struct Keep<P: KeepPersonality> {
     pub kvm_fd: Kvm,
     pub vm_fd: VmFd,
     pub cpu_fds: Vec<VcpuFd>,
-    // FIXME: This will be removed in the near future
-    pub sallyport_start: VirtAddr,
+    pub sallyport_block_size: usize,
     pub sallyports: Vec<Option<VirtAddr>>,
     pub regions: Vec<Region>,
     pub personality: P,
@@ -75,12 +74,12 @@ impl crate::backend::Backend for Backend {
 
     #[inline]
     fn shim(&self) -> &'static [u8] {
-        include_bytes!(concat!(env!("OUT_DIR"), "/bin/shim-sev"))
+        include_bytes!(concat!(env!("OUT_DIR"), "/bin/shim-kvm"))
     }
 
     #[inline]
     fn have(&self) -> bool {
-        data::dev_kvm().pass
+        self.data().iter().all(|x| x.pass)
     }
 
     fn data(&self) -> Vec<super::Datum> {
