@@ -1,6 +1,5 @@
 // SPDX-License-Identifier: Apache-2.0
 
-pub use snp::certs;
 pub use snp::firmware::Firmware;
 
 use super::kvm::mem::Region;
@@ -50,12 +49,12 @@ impl super::Backend for Backend {
 
     #[inline]
     fn shim(&self) -> &'static [u8] {
-        include_bytes!(concat!(env!("OUT_DIR"), "/bin/shim-sev"))
+        include_bytes!(concat!(env!("OUT_DIR"), "/bin/shim-kvm"))
     }
 
     #[inline]
     fn have(&self) -> bool {
-        data::dev_sev_writable().pass
+        self.data().iter().all(|x| x.pass)
     }
 
     fn data(&self) -> Vec<super::Datum> {
