@@ -1,5 +1,6 @@
 // SPDX-License-Identifier: Apache-2.0
 
+mod attestation;
 mod builder;
 mod config;
 mod data;
@@ -19,6 +20,7 @@ use std::sync::{Arc, RwLock};
 struct Tcs;
 
 struct Keep {
+    sallyport_block_size: u64,
     _mem: Map<perms::Unknown>,
     tcs: RwLock<Vec<*const Tcs>>,
 }
@@ -38,7 +40,7 @@ impl crate::backend::Backend for Backend {
 
     #[inline]
     fn have(&self) -> bool {
-        data::dev_sgx_enclave().pass
+        self.data().iter().all(|x| x.pass)
     }
 
     fn data(&self) -> Vec<super::Datum> {
