@@ -6,6 +6,8 @@ mod log;
 mod run;
 #[cfg(feature = "backend-sev")]
 pub mod sev;
+#[cfg(feature = "backend-sgx")]
+pub mod sgx;
 
 use anyhow::{anyhow, Result};
 use std::ops::Deref;
@@ -22,6 +24,8 @@ pub enum Command {
     Run(run::Options),
     #[cfg(feature = "backend-sev")]
     Sev(sev::Command),
+    #[cfg(feature = "backend-sgx")]
+    Sgx(sgx::Command),
 }
 
 //
@@ -46,13 +50,13 @@ impl BackendOptions {
                 .deref()
                 .iter()
                 .find(|b| b.have() && b.name() == name)
-                .ok_or_else(|| anyhow!("Keep backend {:?} is unsupported", name))
+                .ok_or_else(|| anyhow!("Keep backend {:?} is unsupported.", name))
         } else {
-            BACKENDS
-                .deref()
-                .iter()
-                .find(|b| b.have())
-                .ok_or_else(|| anyhow!("No supported backend found"))
+            BACKENDS.deref().iter().find(|b| b.have()).ok_or_else(|| {
+                anyhow!(
+                    "No supported backend found. Please check your machine with `$ enarx info`."
+                )
+            })
         }
         .map(|b| &**b)
     }
