@@ -16,4 +16,9 @@ pub struct Options {
     /// Path of the WebAssembly module to run
     #[structopt(value_name = "MODULE", parse(from_os_str))]
     pub module: PathBuf,
+
+    /// gdb options
+    #[cfg(feature = "gdb")]
+    #[structopt(long, default_value = "localhost:23456")]
+    pub gdblisten: String,
 }
